import axios from 'axios';
import S3 from 'aws-sdk/clients/s3';

var Cache = /*#__PURE__*/function () {
  /**
   * Constructor sets empty values and sets config expiration
   * @param cacheExpiration configuration expiration to create expires value
   */
  function Cache(cacheExpiration) {
    this.store = {};
    this.cacheExpiration = cacheExpiration;
    this.expires = 0;
    this.etag = '';
    this.versionId = '';
  }
  /**
   * Sets data as a local value and creates expires date
   * @param data Payload of the data to store locally
   */


  var _proto = Cache.prototype;

  _proto.set = function set(data, etag, versionId) {
    this.store = data;
    this.expires = new Date().getTime() + this.cacheExpiration;
    this.etag = etag ? etag : '';
    this.versionId = versionId ? versionId : '';
  }
  /**
   * Gets data that is stored in memory
   * @returns locally stored data
   */
  ;

  _proto.get = function get() {
    return this.store;
  }
  /**
   * Gets etag from data that is stored in memory
   * @returns etag of locally stored data
   */
  ;

  _proto.getEtag = function getEtag() {
    return this.etag;
  }
  /**
   * Checks if there is a valid cache
   * @returns true if cache is valid, false if its not
   */
  ;

  _proto.exists = function exists() {
    if (this.store && this.expires > new Date().getTime()) {
      // Checks if there is a stored object, and that has not expired yet
      return true;
    } else {
      // Sets store to default value
      this.clear();
      return false;
    }
  }
  /**
   * Clears local cache
   */
  ;

  _proto.clear = function clear() {
    this.store = {};
    this.expires = 0;
    this.etag = '';
  };

  return Cache;
}();

var HTTP = axios;

var NowAdapter = /*#__PURE__*/function () {
  function NowAdapter(config) {
    this.config = config;
    this.client = HTTP.create({
      baseURL: this.config.instanceUrl,
      timeout: 15000,
      headers: {
        'Content-Type': 'application/json'
      }
    });
  }

  NowAdapter.init = function init(config) {
    return new NowAdapter(config);
  };

  var _proto = NowAdapter.prototype;

  _proto.set = function set(value, path) {
    try {
      var _this2 = this;

      return Promise.resolve(_this2.client.post('/', value, {
        params: {
          path: JSON.stringify(path)
        }
      })).then(function (payload) {
        return payload.data;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.update = function update(value, path) {
    try {
      var _this4 = this;

      return Promise.resolve(_this4.client.put('', value, {
        params: {
          path: JSON.stringify(path)
        }
      })).then(function (payload) {
        return payload.data;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.get = function get(path) {
    try {
      var _this6 = this;

      return Promise.resolve(_this6.client.get('', {
        params: {
          path: JSON.stringify(path)
        }
      })).then(function (payload) {
        return payload.data;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.remove = function remove(path) {
    try {
      var _this8 = this;

      return Promise.resolve(_this8.client["delete"]('', {
        params: {
          path: JSON.stringify(path)
        }
      })).then(function (payload) {
        return payload.data;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return NowAdapter;
}();

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/**
 * Builds a document url out of the array of path item
 * @param path an array of the path, to build final url
 * @returns string of the final path with extension of the document
 */
var documentPath = function documentPath(path) {
  // Needs to be a valid array
  if (!Array.isArray(path)) {
    throw Error('Not a valid path');
  }

  var pathArray = path.map(function (item, index) {
    if (path.length === index + 1 && item.type === 'document') {
      // This is the last item
      return item.key + '.json';
    } else {
      // not the last item
      return item.key;
    }
  });
  return pathArray.join('/');
};
var checkValidObject = function checkValidObject(obj) {
  if (typeof obj === 'object' && obj instanceof Object && !(obj instanceof Array)) {
    return true;
  } else {
    throw Error('Not a valid object');
  }
};
var reservedKeyWords = {
  _meta: true,
  collection: true,
  doc: true,
  indices: true
};
/**
 * Check if key that is used for document or collection is reserved
 * @param keyword to check
 * @returns true if its a reserved key
 */

function isReservedKey(keyword) {
  return reservedKeyWords[keyword];
}
/**
 * Creates a random uuid
 * @returns an uuid
 */

function uuid() {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
    var r = Math.random() * 16 | 0;
    var v = c === 'x' ? r : r & 0x3 | 0x8;
    return v.toString(16);
  });
}

var constants = {
  deleteValue: 'NEARDB.FIELD.DELETE'
};

var S3Adapter = /*#__PURE__*/function () {
  function S3Adapter(config) {
    this.config = config;
    this.client = new S3(this.config.storage);
  }

  S3Adapter.init = function init(config) {
    return new S3Adapter(config);
  };

  var _proto = S3Adapter.prototype;

  _proto.set = function set(value, path) {
    try {
      var _this2 = this;

      var fileKey = documentPath(path);
      var params = {
        Body: JSON.stringify(value),
        Bucket: _this2.config.storage.bucket,
        Key: fileKey
      };
      return Promise.resolve(_this2.client.putObject(params).promise()).then(function () {
        return fileKey;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.update = function update(value, path) {
    try {
      var _this4 = this;

      return Promise.resolve(_this4.get(path)).then(function (doc) {
        // Loop through all property for custom object actions
        for (var prop in value) {
          if (value[prop] === constants.deleteValue) {
            // If has deleteValue action, delete the prop
            delete value[prop];
            delete doc[prop];
          }
        }

        var updatedValue = _extends({}, doc, value);

        return Promise.resolve(_this4.set(updatedValue, path)).then(function () {});
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.get = function get(path) {
    try {
      var _this6 = this;

      var fileKey = documentPath(path);
      var params = {
        Bucket: _this6.config.storage.bucket,
        Key: fileKey
      };
      return Promise.resolve(_this6.client.getObject(params).promise()).then(function (data) {
        return data.Body ? JSON.parse(data.Body.toString('utf8')) : null;
      }); // Convert Buffer to object
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.remove = function remove(path) {
    try {
      var _this8 = this;

      var fileKey = documentPath(path);
      var params = {
        Bucket: _this8.config.storage.bucket,
        Key: fileKey
      };
      return Promise.resolve(_this8.client.deleteObject(params).promise()).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return S3Adapter;
}();

// A type of promise-like that resolves synchronously and supports only one observer

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

var defaultConfig = {
  cacheExpiration: 500
};
var NearDB = /*#__PURE__*/function () {
  /**
   * Constructor to setup config, and path, and required checks.
   * @param config configuration to initialize NearDB instance
   */
  function NearDB(config) {
    // Creates instanceid
    this.instanceId = uuid();
    /** Overwrites config param with default configuration */

    this.config = _extends({}, defaultConfig, config); // TODO: define the type of storage in the config

    this.adapter = new S3Adapter(config);
  }
  /**
   * Static method to create a new instance of NearDB class.
   * @param config configuration to initialize NearDB instance
   * @returns an initialized instance of NearDB with the config
   */


  NearDB.database = function database(config) {
    return new NearDB(Object.assign(defaultConfig, config));
  }
  /**
   * Returns a new instance of collection
   * @param key expects key for collection
   * @returns an instance of the collection class
   */
  ;

  var _proto = NearDB.prototype;

  _proto.collection = function collection(key) {
    return new Collection(this, key);
  };

  return NearDB;
}(); // Constants used for document update

NearDB.field = {
  deleteValue: constants.deleteValue
};
var Collection = /*#__PURE__*/function () {
  function Collection(instance, key, path) {
    // Check if this is a reserved keyword
    if (isReservedKey(key)) {
      throw Error(key + ': is a reserved keyword');
    }

    this.instance = instance; // Copy value of path before passing, to avoid polluting scope

    var newPath = path ? [].concat(path) : [];
    newPath.push({
      type: 'collection',
      key: key
    });
    this.uri = newPath;
  }

  var _proto2 = Collection.prototype;

  _proto2.doc = function doc(key) {
    return new Document(this.instance, key, this.uri);
  }
  /**
   * Adds a document to a collection by generating an id for the doc
   * @param value expects payload to be stored for the document
   * @returns a promise for the payload of the saved doc
   */
  ;

  _proto2.add = function add(value) {
    try {
      var _this2 = this;

      return Promise.resolve(new Document(_this2.instance, uuid(), _this2.uri).set(value));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return Collection;
}();
var Document = /*#__PURE__*/function () {
  /** String path for storage */
  function Document(instance, key, path) {
    // Check if this is a reserved keyword
    if (isReservedKey(key)) {
      throw Error(key + ': is a reserved keyword');
    } // Copy value of path before passing, to avoid polluting scope


    var newPath = [].concat(path); // Push new pathItem into the path array

    newPath.push({
      type: 'document',
      key: key
    });
    this.uri = newPath;
    this.instance = instance; // Sets default cache value

    this.cache = new Cache(this.instance.config.cacheExpiration);
  }
  /**
   * Creates a new instance of Collection
   * @param key expects key for collection
   * @returns an instance of the collection class
   */


  var _proto3 = Document.prototype;

  _proto3.collection = function collection(key) {
    return new Collection(this.instance, key, this.uri);
  }
  /**
   * Sets document data from the path provided in the scope.
   * @param value expects payload to be stored for the document
   * @returns payload of the document requested
   */
  ;

  _proto3.set = function set(value) {
    try {
      var _this4 = this;

      checkValidObject(value);
      return Promise.resolve(_this4.instance.adapter.set(value, _this4.uri));
    } catch (e) {
      return Promise.reject(e);
    }
  }
  /**
   * Gets document data from the path provided in the scope.
   * @param options sets options on how to get documents
   * @returns payload of the document requested
   */
  ;

  _proto3.get = function get() {
    var _this5 = this;

    return Promise.resolve(_catch(function () {
      return Promise.resolve(_this5.instance.adapter.get(_this5.uri));
    }, function () {
      // TODO: log error, but assume doc does not exist
      return null;
    }));
  }
  /**
   * Update some fields of the document without overwriting entire document
   * @param value expects payload to be stored for the document
   * @returns a promise for the payload requested
   */
  ;

  _proto3.update = function update(value) {
    try {
      var _this7 = this;

      checkValidObject(value);
      return Promise.resolve(_this7.instance.adapter.update(value, _this7.uri)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }
  /**
   * Removes document store in the path of instance
   * @returns payload of the document requested
   */
  ;

  _proto3["delete"] = function _delete() {
    try {
      var _this9 = this;

      return Promise.resolve(_this9.instance.adapter.remove(_this9.uri)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return Document;
}();

export { Cache, Collection, Document, HTTP, NearDB, NowAdapter, S3Adapter, uuid };
//# sourceMappingURL=neardb.esm.js.map
